@@ -97,7 +97,7 @@ final class WorkerThread extends XnioIoThread implements XnioExecutor {
 
     static {
         OLD_LOCKING = Boolean.parseBoolean(AccessController.doPrivileged(new ReadPropertyAction("xnio.nio.old-locking", "false")));
-        THREAD_SAFE_SELECTION_KEYS = Boolean.parseBoolean(AccessController.doPrivileged(new ReadPropertyAction("xnio.xnio.thread-safe-selection-keys", "false")));
+        THREAD_SAFE_SELECTION_KEYS = Boolean.parseBoolean(AccessController.doPrivileged(new ReadPropertyAction("xnio.nio.thread-safe-selection-keys", "false")));
     }
 
     WorkerThread(final NioXnioWorker worker, final Selector selector, final String name, final ThreadGroup group, final long stackSize, final int number) {
@@ -259,8 +259,10 @@ final class WorkerThread extends XnioIoThread implements XnioExecutor {
                 if (optionMap.contains(Options.SEND_BUFFER)) channel.socket().setSendBufferSize(optionMap.get(Options.SEND_BUFFER, -1));
                 final SelectionKey key = registerChannel(channel);
                 final NioSocketStreamConnection connection = new NioSocketStreamConnection(this, key, null);
-                channel.socket().bind(bindAddress);
-                ChannelListeners.invokeChannelListener(connection, bindListener);
+                if (bindAddress != null || bindListener != null) {
+                    channel.socket().bind(bindAddress);
+                    ChannelListeners.invokeChannelListener(connection, bindListener);
+                }
                 if (channel.connect(destinationAddress)) {
                     execute(ChannelListeners.getChannelListenerTask(connection, openListener));
                     final FinishedIoFuture<StreamConnection> finishedIoFuture = new FinishedIoFuture<StreamConnection>(connection);
@@ -419,6 +421,8 @@ final class WorkerThread extends XnioIoThread implements XnioExecutor {
         }
     }
 
+    volatile boolean polling;
+
     public void run() {
         final Selector selector = this.selector;
         try {
@@ -459,6 +463,8 @@ final class WorkerThread extends XnioIoThread implements XnioExecutor {
                             task = workQueue.poll();
                         }
                     }
+                    // clear interrupt status
+                    Thread.interrupted();
                     safeRun(task);
                 } while (task != null);
                 // all tasks have been run
@@ -492,6 +498,8 @@ final class WorkerThread extends XnioIoThread implements XnioExecutor {
                     }
                     Arrays.fill(keys, 0, keys.length, null);
                 }
+                // clear interrupt status
+                Thread.interrupted();
                 // perform select
                 try {
                     if ((oldState & SHUTDOWN) != 0) {
@@ -499,11 +507,29 @@ final class WorkerThread extends XnioIoThread implements XnioExecutor {
                         selector.selectNow();
                     } else if (delayTime == Long.MAX_VALUE) {
                         selectorLog.tracef("Beginning select on %s", selector);
-                        selector.select();
+                        polling = true;
+                        try {
+                            if (workQueue.peek() != null) {
+                                selector.selectNow();
+                            } else {
+                                selector.select();
+                            }
+                        } finally {
+                            polling = false;
+                        }
                     } else {
                         final long millis = 1L + delayTime / 1000000L;
                         selectorLog.tracef("Beginning select on %s (with timeout)", selector);
-                        selector.select(millis);
+                        polling = true;
+                        try {
+                            if (workQueue.peek() != null) {
+                                selector.selectNow();
+                            } else {
+                                selector.select(millis);
+                            }
+                        } finally {
+                            polling = false;
+                        }
                     }
                 } catch (CancelledKeyException ignored) {
                     // Mac and other buggy implementations sometimes spits these out
@@ -536,6 +562,8 @@ final class WorkerThread extends XnioIoThread implements XnioExecutor {
                             if (handle == null) {
                                 cancelKey(key);
                             } else {
+                                // clear interrupt status
+                                Thread.interrupted();
                                 handle.handleReady(key.readyOps());
                             }
                         }
@@ -570,7 +598,7 @@ final class WorkerThread extends XnioIoThread implements XnioExecutor {
         synchronized (workLock) {
             selectorWorkQueue.add(command);
         }
-        if(currentThread() != this) {
+        if (polling) { // flag is always false if we're the same thread
             selector.wakeup();
         }
     }
@@ -605,7 +633,7 @@ final class WorkerThread extends XnioIoThread implements XnioExecutor {
             queue.add(key);
             if (queue.iterator().next() == key) {
                 // we're the next one up; poke the selector to update its delay time
-                if(currentThread() != this) {
+                if (polling) { // flag is always false if we're the same thread
                     selector.wakeup();
                 }
             }
@@ -665,7 +693,7 @@ final class WorkerThread extends XnioIoThread implements XnioExecutor {
             try {
                 return channel.register(selector, 0);
             } finally {
-                selector.wakeup();
+                if (polling) selector.wakeup();
             }
         } else {
             final SynchTask task = new SynchTask();
@@ -744,7 +772,7 @@ final class WorkerThread extends XnioIoThread implements XnioExecutor {
         } else {
             try {
                 key.interestOps(key.interestOps() | ops);
-                selector.wakeup();
+                if (polling) selector.wakeup();
             } catch (CancelledKeyException ignored) {
             }
         }
